
# biolinux bash shell script with variables and function; to be run in same directory as BLAT executable.
# Direct read mapping using pre-existing 20 super-references.
# Creates numerous temporary files overwritten as each sample is processed; echos some information to screen during run.
# Output files: _BCstats.txt; _Ex3numbers.txt; _reads.txt; _SR20t50dists.txt; _across.fasta. Plus optional: _stats.txt; _table7.txt
# Set-up, convert to fasta; find best SR using 20stest50 and first 50k reads; do BLAT mapping using: 1552stNDSR.fasta file, 285165 sites; block number 3, match 35, size 15 limits; base calling; summary stats.
# Note settings for: test exons file and read subset for selecting best SR; BLAT stringenies; SR file; total length; block number, match and size limits; coverage and polymorphism limits.


pslxtallier(){
echo unzipping $NAME

# FORMAT APPROPRIATE DIRECTORY PATHS AND READ FILE LABELS
gunzip -c /home/ahugall/blatSuite/TREADS/${NAME}"_T820_R1.fastq.gz" > sample.fastq
gunzip -c /home/ahugall/blatSuite/TREADS/${NAME}"_T820_R2.fastq.gz" >> sample.fastq

perl -e '$i=0;while(<>){if(/^\@/&&$i==0){s/^\@/\>/;print;}elsif($i==1){print;$i=-3}$i++;}' sample.fastq > sample.fasta

echo ">"${NAME}"_"${SETS} > label.txt
cat label.txt >> ${RUN}"_"${SETS}"_BCstats.txt"
cat label.txt >> ${RUN}"_"${SETS}"_Ex3numbers.txt"
grep '>' sample.fasta -c > reads.txt
paste label.txt reads.txt | cat >> ${RUN}"_"${SETS}"_reads.txt"

echo checking SRs with first 50k reads and 20stest50 set of 50 exons
head -n 100000 sample.fasta > check.fasta
./blat check.fasta 20stest50.fasta check50.psl -t=dna -q=dna -minIdentity=87 -out=psl
tail -n +6 check50.psl | cut -f 10 | uniq -c | sort -nr > Ex3hits.txt
cat Ex3hits.txt >> ${RUN}"_"${SETS}"_SRuse.txt"
head -n 1 Ex3hits.txt | awk '{print $NF}' > SR.txt
grep -f SR.txt check50.psl > these.txt
awk '{ s+=$1; m+=$2} END {print s, m, m/(s+m)}' these.txt > those.txt
paste label.txt SR.txt those.txt >> ${RUN}"_SR20t50dists.txt"

# separates exons
sed -n 2p 1552stNDSR.fasta | sed 's/\(.\)/\1\n/g' > tagline.txt
grep -A 1 -f SR.txt 1552stNDSR.fasta > BSR.fasta
sed -n 2p BSR.fasta | sed 's/\(.\)/\1\n/g' > seq1.txt
paste tagline.txt seq1.txt > seq2.txt
awk '{ if ($1=="|") printf("\n%s", $2); else printf($2)}' seq2.txt | tail -n 1552 > c1.txt
paste 1552_exons_start_list.txt c1.txt | tr '\t' '\n' > BSR.fasta

echo -n doing BLAT -minIdentity=87 mapping with bestSR_
cat SR.txt

# SET BLAT MAPPING STRINGENY HERE - pslx output includes matched sequence blocks
./blat sample.fasta BSR.fasta BSR_sample_cov.pslx -t=dna -q=dna -minIdentity=87 -out=pslx
# ./blat sample.fasta BSR.fasta BSR_sample_cov.pslx -t=dna -q=dna -minIdentity=92 -oneOff=1 -out=pslx

# contains block number, size and gap length limits
echo parsing the BLAT output
tail -n +6 BSR_sample_cov.pslx | tr '@' ' ' > BLATout.txt
awk '{if ($1>35 && $19<4 && $9=="+") print $11, $12, $19, $20, $21, $24}' BLATout.txt > plusMB.pslx
awk '{if ($1>35 && $19<4 && $9=="-") print $11, $12, $19, $20, $21, $24}' BLATout.txt > minusMB.pslx
tr "," " " < minusMB.pslx | tr -d '\t' | tr -s " " > minusMB.txt
tr "," " " < plusMB.pslx | tr -d '\t' | tr -s " " > plusMB.txt
awk '{for (b=1; b<=$3; b++) if ($(3+b)>17) print $(3+$3+b)+$1, $(3+$3*2+b)}' plusMB.txt > plusN.txt
awk '{for (b=1; b<=$3; b++) if ($(4+$3-b)>17) print $1+$2-$(4+$3-b)-$(4+$3*2-b), $(4+$3*3-b)}' minusMB.txt > minusN.txt
cut -d ' ' -f 1 plusN.txt > plusP.txt
cut -d ' ' -f 1 minusN.txt > minusP.txt
cut -d ' ' -f 2 plusN.txt > plusS.txt
cut -d ' ' -f 2 minusN.txt > minusS.txt
sed 's/a/ 1/g' plusS.txt | sed 's/c/ 2/g' | sed 's/g/ 3/g' | sed 's/t/ 4/g' | sed 's/n/ 5/g' > plusS4.txt
rev minusS.txt > reverse.txt
sed 's/a/ 4/g' reverse.txt | sed 's/c/ 3/g' | sed 's/g/ 2/g' | sed 's/t/ 1/g' | sed 's/n/ 5/g' > minusS4.txt
paste plusP.txt plusS4.txt >  plusPS4.txt
paste minusP.txt minusS4.txt >  minusPS4.txt
cat plusPS4.txt minusPS4.txt > PS4.txt

echo tallying the big site table
awk 'BEGIN { for (i=1; i<=285165; i++) {table[i,1]=0; table[i,2]=0; table[i,3]=0; table[i,4]=0; table[i,5]=0; table[i,6]=i}} {for (fdn=2; fdn<=NF; fdn++) table[($1+fdn-2),$fdn]++} END { for (i=1; i<=285165; i++) print (table[i,1], table[i,2], table[i,3], table[i,4], table[i,5], table[i,6])}' PS4.txt > table6S4T.txt
awk '{ print ($1+$2+$3+$4, $1, $2, $3, $4, $5, $6)}' table6S4T.txt > table7.txt

# contains coverage 5 and polymorphism 0.2 limits
echo calling the bases
awk '{g=0; if($1==0) print "0"; else if($1<5) print"1"; else for(i=2; i<=6; i++) if(i==6) print g*1; else if($i/$1>0.2) g=g i}' table7.txt > table7H.txt
sed 's/0/-/' table7H.txt | sed 's/1/n/' | sed 's/2345/X/'  | sed 's/234/X/' | sed 's/235/X/' | sed 's/245/X/' | sed 's/345/X/' | sed 's/23/M/' | sed 's/24/R/' | sed 's/25/W/' | sed 's/34/S/' | sed 's/35/Y/' | sed 's/45/K/' | sed 's/2/A/' | sed 's/3/C/' | sed 's/4/G/' | sed 's/5/T/' > down.txt

echo summarizing total site composition stats
awk '{if ($1>35 && $19<4) print $1+$2, $15$9}' BLATout.txt > hits.txt
cut -d ' ' -f 1 hits.txt | paste -sd+ | bc >> ${RUN}"_"${SETS}"_BCstats.txt"
sort -u -k2,2 hits.txt | wc -l >> ${RUN}"_"${SETS}"_BCstats.txt"
awk '{if ($1>4) print $1}' table7.txt | wc -l > cov.txt
cat cov.txt >> ${RUN}"_"${SETS}"_BCstats.txt"
sort table7H.txt | uniq -c >> ${RUN}"_"${SETS}"_BCstats.txt"
awk '{printf "%i", 100*$1/285165}' cov.txt > covP.txt
paste -d_ label.txt SR.txt covP.txt > label2.txt
tr -d '\n' < down.txt > across.txt
cat label2.txt across.txt >> ${RUN}"_"${SETS}"_across.fasta"
echo >> ${RUN}"_"${SETS}"_across.fasta"
cat label2.txt

# REMOVE COMMENT HASH TO SAVE SITE COMPOSITION TABLE
# cat table7.txt > /home/ahugall/blatSuite/7tables/${NAME}"_"${SETS}"_table7.txt"

# APPENDS EXON STATS
echo exon stats
awk 'BEGIN{m=0; p=0} NR==FNR{exon[NR]=$1; next} {for (n=$3; n<=$4; n++) if(exon[n]<2) m++; else if(exon[n]>10) p++} {print $2-m,p} {m=0; p=0}' table7H.txt 1552exons.txt > exon_stats.txt
awk 'BEGIN{m=0} NR==FNR{exon[NR]=$1; next} {for(n=$3; n<=$4; n++) m=m+exon[n]} {printf "%.1f\n", (m+exon[n])/$2} {m=0}' table7.txt 1552exons.txt > exon_cov.txt
paste ${RUN}"_"${SETS}"_stats.txt" exon_stats.txt exon_cov.txt > exon_stats2.txt
cat exon_stats2.txt > ${RUN}"_"${SETS}"_stats.txt"

echo
}

# INFO TO BE ADDED TO FILE AND SAMPLE LABELS
RUN=OPH07d
SETS=m87c5p2

# REMOVE COMMENT HASH TO CREATE NEW EXON STATS FILE
# cat 1552exons.txt > ${RUN}"_"${SETS}"_stats.txt"


# INSERT LIST OF SAMPLE NAMES TO BE PROCESSED (as shell variable 'NAME', then invoke shell function)
NAME=Ophiocoma_cynthiae_emilie3
pslxtallier
NAME=Ophiocoma_krohi_emilie2
pslxtallier
NAME=Ophiocoma_sp_emilie7
pslxtallier

