
# biolinux bash shell script with variables and function; to be run in same directory as BLAT executable.
# Creates numerous temporary files overwritten as each sample is processed.
# Reads site composition tables from both TASSER contig mapping and TASSMAP read mapping; blocks out exons with polymorphic sites in excess of pre-set limit. Appends percent of target blocked to sequence label.
# Output files: _HEX_across.fasta (revised sequences); _HEX_BCstats.txt (summary of sites blocked); _HEX_list.txt (exons blocked per sample).
# Note settings for: consensus base calling target size, coverage and polymorphism; excess polymorphism limit.

extatter(){
# SAMPLE LABEL INFO FROM TASSMAP OUTPUT
grep ">"${NAME}"_"${REM} ${RUN}"_TASSMAP_across.fasta" > label.txt
cat label.txt >> ${RUN}"_HEX_list.txt"

# FORMAT APPROPRIATE DIRECTORY PATHS AND FILE LABELS
echo summarizing TASSER exons in folder 7ASR

awk '{ if ($1==0) print "0"; else for (i=2; i<=6; i++) if (i==6) printf "\n"; else if ($i>0) printf "%s", i}' /home/ahugall/blatSuite/7ASR/${NAME}"_"${TASS}"_table7.txt" > table7H.txt
awk 'BEGIN{m=0; p=0} NR==FNR{exon[NR]=$1; next} {for (n=$3; n<=$4; n++) if(exon[n]<2) m++; else if(exon[n]>10) p++} {print $2-m,p} {m=0; p=0}' table7H.txt 1552exons.txt > TASSexstat.txt

# contains coverage 5 and polymorphism 0.20 limits
echo summarizing TASSMAP exons in folder 7tables
awk '{g=0; if($1==0) print "0"; else if($1<5) print"1"; else for(i=2; i<=6; i++) if(i==6) print g*1; else if($i/$1>0.20) g=g i}' /home/ahugall/blatSuite/7tables/${NAME}"_"${REM}"_table7.txt" > table7H.txt
awk 'BEGIN{m=0; p=0} NR==FNR{exon[NR]=$1; next} {for (n=$3; n<=$4; n++) if(exon[n]<2) m++; else if(exon[n]>10) p++} {print $2-m,p} {m=0; p=0}' table7H.txt 1552exons.txt > REMexstat.txt

paste 1552exons.txt TASSexstat.txt REMexstat.txt > ASRexstat.txt

# RECODE SITE CALLS - excess pm limit 0.04
awk 'NR==FNR{site[NR]=$1; next} {for (n=$3; n<=$4; n++) if($7==0 || $5==0) print site[n]; else if($6/$5>0.04 && $8/$7>0.04) print 2345; else print site[n]}' table7H.txt ASRexstat.txt > table7HEX.txt
sed 's/0/-/' table7HEX.txt | sed 's/1/n/' | sed 's/2345/x/'  | sed 's/234/X/' | sed 's/235/X/' | sed 's/245/X/' | sed 's/345/X/' | sed 's/23/M/' | sed 's/24/R/' | sed 's/25/W/' | sed 's/34/S/' | sed 's/35/Y/' | sed 's/45/K/' | sed 's/2/A/' | sed 's/3/C/' | sed 's/4/G/' | sed 's/5/T/' > down.txt

# OUTPUT STATS
awk '{if($1==2345) n++} END{print n}' table7HEX.txt > cov.txt
awk '{printf "%.1f", 100*$1/285165}' cov.txt > covP.txt
awk 'BEGIN{s=0;n=0} {if($7==0 || $5==0){} else if($6/$5>0.04 && $8/$7>0.04) {s+=$2; n++}} END{print n,s}' ASRexstat.txt > exex.txt
paste label.txt exex.txt | cat >> ${RUN}"_HEXstats.txt"
awk '{if($7==0 || $5==0); else if($6/$5>0.04 && $8/$7>0.04) print $1,$2,$6/$5,$8/$7}' ASRexstat.txt > exex.txt
cat exex.txt >> ${RUN}"_HEXlist.txt"

paste -d_ label.txt covP.txt > label2.txt
tr -d '\n' < down.txt > across.txt
cat label2.txt across.txt >> ${RUN}"_HEX_across.fasta"
echo >> ${RUN}"_HEX_across.fasta"
cat label2.txt
}


# RUN LABEL AND TASSER (TASS) AND TASSMAP (REM) TABLE7 SET LABELS
RUN=OPH07d
TASS=eTASSf
REM=t93c5p2

# INSERT LIST OF SAMPLE NAMES TO BE PROCESSED (as shell variable 'NAME', then invoke shell function)
NAME=Ophiocoma_cynthiae_emilie3
extatter
NAME=Ophiocoma_krohi_emilie2
extatter
NAME=Ophiocoma_sp_emilie7
extatter
