
# biolinux bash shell script with variables and function; to be run in same directory as BLAT executable. Creates ASR reference file for TASSMAP.
# Creates numerous temporary files overwritten as each sample is processed. This version selects closest SR using contigs.
# Output files: _TASS_summary.txt _Trin_across.fasta. Plus optional: _TASS_stats.txt; _table7.txt
# Trinity assembly contig mapper; select best SR using test50AAY.fasta file, BLAT AA map minIdentity=86, 1552sNDSRAAY.fasta reference file; all matching contigs, separate exons, min match 17 AA, min block 5 AA, max Qgap 50 AA
# Note settings for: test exons file and read subset for selecting best SR; BLAT stringenies; SR file; total length; block number, match and size, and Qgap length limits; coverage and polymorphism limits


tassconpiler(){
# FORMAT APPROPRIATE DIRECTORY PATHS AND TRINTY ASSMBLY FILE LABEL
cat /home/ahugall/blatSuite/EC_TASS/${NAME}".Trinity.fasta" > sample.Trinity.fasta
echo ">"${NAME}"_"${SETS} > label.txt

awk 'BEGIN { FS = "len=" } {print($1)}' sample.Trinity.fasta > trin1.txt
tr -d '\n' < trin1.txt | tr '>' '\n' | tail -n +2 | sort -k1,1 > trin4S.txt
./blat sample.Trinity.fasta 20stest50AAY.fasta check50AAY.psl -t=dnax -q=prot -minIdentity=87 -out=psl
tail -n +6 check50AAY.psl | cut -f 10 | sort | uniq -c | sort -nr > Ex3hits.txt
head -n 1 Ex3hits.txt | awk '{print $NF}' > SR.txt
grep -f SR.txt check50AAY.psl > these.txt
awk '{ s+=$1; m+=$2} END {print s, m, m/(s+m)}' these.txt > those.txt

# use predetermined SR
# echo sr16s > SR.txt

sed -n 2p 1552sNDSRAAY.fasta | sed 's/\(.\)/\1\n/g' > tagline.txt
grep -A 1 -f SR.txt 1552sNDSRAAY.fasta | tail -n 1 | sed 's/\(.\)/\1\n/g' > seq1.txt
paste tagline.txt seq1.txt > seq2.txt
awk '{ if ($1=="|") printf("\n%s", $2); else printf($2)}' seq2.txt | tail -n 1552 > c1.txt
paste 1552_exons_start_list.txt c1.txt | tr '\t' '\n' > BSRAAY.fasta

echo -n doing BLAT -minIdentity=87 AA separate exon mapping with bestSR_
cat SR.txt

# SET BLAT MAPPING STRINGENY HERE - amino acid matching
./blat sample.Trinity.fasta BSRAAY.fasta BSRAAY_sample_cov.psl -t=dnax -q=prot -minIdentity=87 -out=psl
# ./blat sample.Trinity.fasta BSRAAY.fasta BSRAAY_sample_cov.psl -t=dnax -q=prot -minIdentity=86 -oneOff=1 -out=psl

tail -n +6 BSRAAY_sample_cov.psl > BLATout.txt
sort -k14,14 BLATout.txt > BLAToutS.txt
join -1 14 -2 1 BLAToutS.txt trin4S.txt  | tr '@' ' ' > BLAToutSS.txt

# contains block number, match and size, and gap length limits
awk '{if ($2>17 && $19<9 && $10=="++" && $7<50) print $12, $14, $16, $17, $18, $19, $20, $21, $22, $23}' BLAToutSS.txt > plusMB.pslx
awk '{if ($2>17 && $19<9 && $10=="+-" && $7<50) print $12, $14, $16, $17, $18, $19, $20, $21, $22, $23}' BLAToutSS.txt > minusMB.pslx
tr "," " " < minusMB.pslx | tr -s " " > minusMB.txt
tr "," " " < plusMB.pslx | tr -s " " > plusMB.txt
awk '{for (b=1; b<=$6; b++) if ($(7+b-1)*3>15) print $(7+b-1+$6)*3+$1, substr($(7+3*$6), $(7+b-1+$6*2)+1, $(7+b-1)*3)}' plusMB.txt > plusB.txt
awk '{for (b=1; b<=$6; b++) if ($(7+b-1)*3>15) print $(7+b-1+$6)*3+$1, substr($(7+3*$6), $3-$(7+b-1+$6*2)-$(7+b-1)*3+1, $(7+b-1)*3)}' minusMB.txt > minusB.txt
cut -d ' ' -f 1 plusB.txt > plusP.txt
cut -d ' ' -f 2 plusB.txt > plusS.txt
sed 's/A/ 1/g' plusS.txt | sed 's/C/ 2/g' | sed 's/G/ 3/g' | sed 's/T/ 4/g' | sed 's/N/ 5/g' > plusS4.txt
paste plusP.txt plusS4.txt > plusPS4.txt
cut -d ' ' -f 1 minusB.txt > minusP.txt
cut -d ' ' -f 2 minusB.txt > minusS.txt
rev minusS.txt > reverse.txt
sed 's/A/ 4/g' reverse.txt | sed 's/C/ 3/g' | sed 's/G/ 2/g' | sed 's/T/ 1/g' | sed 's/N/ 5/g' > minusS4.txt
paste minusP.txt minusS4.txt >  minusPS4.txt
cat plusPS4.txt minusPS4.txt > PS4.txt

echo tallying the big site table
awk 'BEGIN { for (i=1; i<=285165; i++) {table[i,1]=0; table[i,2]=0; table[i,3]=0; table[i,4]=0; table[i,5]=0; table[i,6]=i}} {for (fdn=2; fdn<=NF; fdn++) table[($1+fdn-2),$fdn]++} END { for (i=1; i<=285165; i++) print (table[i,1], table[i,2], table[i,3], table[i,4], table[i,5], table[i,6])}' PS4.txt > table6S4T.txt
awk '{ print ($1+$2+$3+$4, $1, $2, $3, $4, $5, $6)}' table6S4T.txt > table7.txt

echo calling the bases
awk '{ if ($1==0) print "0"; else for (i=2; i<=6; i++) if (i==6) printf "\n"; else if ($i>0) printf "%s", i}' table7.txt > table7H.txt
sed 's/0/-/' table7H.txt | sed 's/1/n/' | sed 's/2345/X/'  | sed 's/234/X/' | sed 's/235/X/' | sed 's/245/X/' | sed 's/345/X/' | sed 's/23/M/' | sed 's/24/R/' | sed 's/25/W/' | sed 's/34/S/' | sed 's/35/Y/' | sed 's/45/K/' | sed 's/2/A/' | sed 's/3/C/' | sed 's/4/G/' | sed 's/5/T/' > down.txt

echo summarizing total site composition stats
awk '{if ($1>0) print $1}' table7.txt | wc -l > cov.txt
awk '{printf "%i", 100*$1/285165}' cov.txt > covP.txt
awk '{ if ($1>10) print $0 }' table7H.txt | wc -l > pm.txt
awk '{printf "%.1f", 100*$1/285165}' pm.txt > pmP.txt
awk '{if ($1>17 && $18<9) print $1+$2, $15}' BLATout.txt | sort -u -k2,2 | wc -l > contigs.txt
paste label.txt SR.txt those.txt contigs.txt cov.txt pm.txt > tassum.txt
cat tassum.txt >> ${RUN}"_TASSummary.txt"

paste -d_ label.txt SR.txt covP.txt pmP.txt > label2.txt
tr -d '\n' < down.txt > across.txt
cat label2.txt across.txt >> ${RUN}"_Trin_across.fasta"
echo >> ${RUN}"_Trin_across.fasta"
cat label2.txt

# REMOVE COMMENT HASH TO SAVE SITE COMPOSITION TABLE
# cat table7.txt > /home/ahugall/blatSuite/7ASR/${NAME}"_"${SETS}"_table7.txt"

# APPENDS EXON STATS
echo exon stats
awk 'BEGIN{m=0; p=0} NR==FNR{exon[NR]=$1; next} {for (n=$3; n<=$4; n++) if(exon[n]<2) m++; else if(exon[n]>10) p++} {print $2-m,p} {m=0; p=0}' table7H.txt 1552exons.txt > exon_stats.txt
paste ${RUN}"_TASS_stats.txt" exon_stats.txt > exon_stats2.txt
cat exon_stats2.txt > ${RUN}"_TASS_stats.txt"
echo
}

# INFO TO BE ADDED TO FILE AND SAMPLE LABELS
RUN=OPH07d
SETS=eTASSf

# REMOVE COMMENT HASH TO CREATE NEW EXON STATS FILE
# cat 1552exons.txt > ${RUN}"_TASS_stats.txt"

# INSERT LIST OF SAMPLE NAMES TO BE PROCESSED (as shell variable 'NAME', then invoke shell function)
NAME=Ophiocoma_cynthiae_emilie3
tassconpiler
NAME=Ophiocoma_krohi_emilie2
tassconpiler
NAME=Ophiocoma_sp_emilie7
tassconpiler

