
# biolinux bash shell script with variables and function; to be run in same directory as BLAT executable.
# Trinity Assembly COI ALL contigs aligned, BLAT AA matched using COISRAAw2.fasta with 31 COI AA sequences. Note uses generic genetic code not mitochondrial code
# Selects all contigs matching COI reference file, trimmed to match length (by codon). Sequence label comprises: sample NAME/contig label/match length
# COI output alignment set to 1431 base length used in reference file COISRAAw2.fasta. Note does not align indels.

trawler(){
# FORMAT APPROPRIATE DIRECTORY PATHS AND TRINTY ASSMBLY FILE LABEL
cat /home/ahugall/blatSuite/EC_TASS/${NAME}".Trinity.fasta" > sample.Trinity.fasta

# SET BLAT MAPPING STRINGENY HERE - amino acid matching
./blat sample.Trinity.fasta COISRAAw2.fasta Trin_ESAAY_cov85.psl -t=dnax -q=prot -minIdentity=85 -out=psl -oneOff=1

awk 'BEGIN { FS = "len=" } {print($1)}' sample.Trinity.fasta > trin1.txt
tr -d '\n' < trin1.txt | tr '>' '\n' | tail -n +2 | sort -k1,1 > trin4S.txt
tail -n +6 Trin_ESAAY_cov85.psl | sort -k14,14 -k1,1r | sort -u -k14,14  > coicont.txt
join -1 14 -2 1 coicont.txt trin4S.txt | sort -k15,15gr > best_BLAToutS.pslx
awk '{print ($1, $2, $5, $6, $9, $10, $11, $12, $13, $14, $15, $16, $17, substr($22,$16+1,$17-$16))}' best_BLAToutS.pslx > best_BLAToutS14.pslx
awk '$6 == "++"'  best_BLAToutS14.pslx > best_plusS14.pslx
awk '$6 == "+-"'  best_BLAToutS14.pslx> best_minusS14.pslx
cut -d ' ' -f 14 best_minusS14.pslx | rev > reverse.txt
sed 's/A/1/g' reverse.txt | sed 's/C/2/g' | sed 's/G/3/g' | sed 's/T/4/g' > reverse1.txt
sed 's/1/T/g' reverse1.txt | sed 's/2/G/g' | sed 's/3/C/g' | sed 's/4/A/g' > minus_seq_revcomp.txt
awk '{$14="";print}' best_minusS14.pslx > best_minusS13.pslx
paste best_minusS13.pslx minus_seq_revcomp.txt > best_minusS15.pslx
cat best_minusS15.pslx best_plusS14.pslx > best14.pslx
awk '{ for (i=1; i <= $9*3; i++) printf "~"; print "@"}' best14.pslx > leadS.txt
awk '{ for (i=1; i <= ($8-$10)*3; i++) printf "~"; print "@"}' best14.pslx > trailS.txt
paste best14.pslx leadS.txt trailS.txt | sort -k2,2gr | awk '{ print(">""'$NAME'""_"$1"_"($13-$12), $15 $14 $16)}' | tr -d '@' | tr ' ' '\n' >> ${RUN}"_COI_contigs.fasta"
echo
}

# PREFIX TO BE ADDED TO OUTPUT FILE LABEL
RUN=OPH07d

# INSERT LIST OF SAMPLE NAMES TO BE PROCESSED (as shell variable 'NAME', then invoke shell function)
NAME=Ophiocoma_cynthiae_emilie3
trawler
NAME=Ophiocoma_krohi_emilie2
trawler
NAME=Ophiocoma_sp_emilie7
trawler

